<?php
// Generate a single day image
// Called by a post from the index.php form

//---------------------------------------------
function FitTextToSize($AvailableWidth,$AvailableHeight,$Font,$Text){
// Return an array of the
//  [0] largest font size that can be accomodated in the dimensions
//  [1] Width
//  [2] Height
//---------------------------------------------

  $size = 40;
  $previousSize = 0;
  for($i=0;$i<5;$i++){   // use this to stop any possibility of infinite jiggling
                         // between too big and too small
                         
    if($size==$previousSize){break;}
    $previousSize = $size;                     
    $a = imagettfbbox($size,0,$Font,$Text);         
    $boxHeight = $a[1] - $a[5];
    $boxWidth  = $a[4] - $a[0];
    if(($boxHeight<3) or ($boxWidth<3)){die ("<br>neg box size S=$size W=$boxWidth H=$boxHeight");}
    
    $widthScale  = $AvailableWidth  / $boxWidth;     // eg 500 / 400 = 1.25 => can be larger
    $heightScale = $AvailableHeight / $boxHeight;    // eg 50 / 60 = .83 => must be smaller
    if($widthScale<1){  // too wide
      $size = ($size * $widthScale)-1;
      continue;
    }
    if($heightScale<1){  // too high
      $size = ($size * $heightScale)-1;
      continue;
    }
    $enlargeBy = ($widthScale<$heightScale) ? $widthScale : $heightScale;
    if($enlargeBy<1.05){break;}
    $size = ($size * $enlargeBy) -1;
  }
  $a = imagettfbbox($size,0,$Font,$Text);
  $boxHeight = $a[1] - $a[5];
  $boxWidth  = $a[4] - $a[0];
  return array($size,$boxWidth,$boxHeight);  
}  
  
// arry used to convert colour names to RGB
$rgbs = array();
$rgbs['Black']=array(0,0,0);
$rgbs['Blue']=array(0,0,255);
$rgbs['Bright green']=array(0,255,0);
$rgbs['Brown']=array(64,0,0);
$rgbs['Dark grey']=array(64,64,64);
$rgbs['Dark green']=array(0,128,0);
$rgbs['Grey']=array(128,128,128);
$rgbs['Grey-green']=array(0,128,128);
$rgbs['Light blue']=array(0,255,255);
$rgbs['Pink']=array(255,128,160);
$rgbs['Purple']=array(255,0,255);
$rgbs['Red']=array(255,0,0);
$rgbs['Silver']=array(192,192,192);
$rgbs['White']=array(255,255,255);
$rgbs['Yellow']=array(255,255,0);

$sizes = array();
$sizes['Small 100px']=100;
$sizes['Medium 200px']=200;
$sizes['Large 400px']=400;
$sizes['Huge 1000px']=1000;


//---------------------------------------------
function ConvertColourNameToRGB($Image,$ColourName){
// Given an input from a select box, return an array of R,G,B
//---------------------------------------------
  global $rgbs;
  if(!array_key_exists($ColourName,$rgbs)){
    return imagecolorallocate($Image, 16,32,64);
  }else{  
    $rgb = $rgbs[$ColourName];
    return imagecolorallocate($Image,$rgb[0],$rgb[1],$rgb[2]);
  }
}


// ******************************
// *** read the POSTed values ***
// ******************************
extract($_POST);
// dimensions
$width = array_key_exists($sizeOpt,$sizes) ? $sizes[$sizeOpt] : 200;
$height = $width * 1.5;
$headHeight = $height * 0.3;
$roundSize = $headHeight *.3;
$font = "ariblk.ttf";

// create the image to draw on
$img = imageCreate($width,$height);
$white = imagecolorallocate($img, 255, 255, 255);  // set bg white as well
imageFillToBorder($img, 5,5,$white,$white);

// tidy texts
$monthText = trim($monthText);
$monthText = ($monthText=='') ? '???' : $monthText;
$monthText = substr($monthText,0,20);

$eventText = trim($eventText);
$eventText = substr($eventText,0,20);

$dayNumText = trim($dayNumText);
$dayNumText = ($dayNumText=='') ? '??' : $dayNumText;
$dayNumText = substr($dayNumText,0,2);

$dayTextText = trim($dayTextText);
$dayTextText = substr($dayTextText,0,20);


// look up colours
$colourH = ConvertColourNameToRGB($img,$headColour);
$colourM = ConvertColourNameToRGB($img,$monthColour);
$colourE = ConvertColourNameToRGB($img,$eventColour);
$colourN = ConvertColourNameToRGB($img,$dayNumColour);
$colourD = ConvertColourNameToRGB($img,$dayTextColour);

// *******************************
// *** Draw the image elements ***
// *******************************




// draw bottom rectangle
imageSetThickness($img,1);
imageantialias ($img,true);
imageLine($img, 2,$headHeight,2,$height,$colourN);
imageLine($img, 2,$height-2,$width-2,$height-2,$colourN);
imageLine($img, $width-2,$height-2,$width-2,$headHeight,$colourN);
imageLine($img, $width-2,$headHeight,2,$headHeight,$colourH);
// draw head outline
imageLine($img, 2,$headHeight,2,$roundSize,$colourH);
imageLine($img, $width-2,$headHeight,$width-2,$roundSize,$colourH);
imageLine($img, $roundSize,2,$width-$roundSize,2,$colourH);
imageArc( $img, $roundSize+2,$roundSize+2,$roundSize*2,$roundSize*2,180,270,$colourH);
imageArc( $img, $width-$roundSize-2,$roundSize+2,$roundSize*2,$roundSize*2,270,0,$colourH);
// fill head
imageFillToBorder($img, $roundSize,$roundSize,$colourH,$colourH);

// Month name in head
// ------------------
$a = FitTextToSize($width * .8,$headHeight * .8,$font,$monthText);
imagettftext($img, $a[0], 0, ($width - $a[1])/2,$headHeight*0.80, $colourM, $font, $monthText);



// Event (optional)
if($eventText){
  $a = FitTextToSize($width * .9,($height - $headHeight) * .2,$font,$eventText);
  imagettftext($img, $a[0], 0, (($width*1) - $a[1])/2,$headHeight+($a[2]*1.1), $colourE, $font, $eventText);
}
// Day name (etc) on bottom of block
if($dayTextText){
  $a = FitTextToSize($width * .9,($height - $headHeight) * .2,$font,$dayTextText);
  imagettftext($img, $a[0], 0, (($width*1) - $a[1])/2,$height*0.95, $colourD, $font, $dayTextText);
}

// Day number in block
$a = FitTextToSize($width * .8,($height - $headHeight) * .6,$font,$dayNumText);
imagettftext($img, $a[0], 0, (($width*.9) - $a[1])/2,$height*0.85, $colourN, $font, $dayNumText);


header("Content-type: image/png");
imagepng($img);
imagedestroy($img);

?>